﻿using System;
using System.ServiceProcess;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace MyService
{
	public partial class MyService : ServiceBase
	{
		static String startPath = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "MyStart.bat";
		static String stopPath = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "MyStop.bat";

		public MyService()
		{
			InitializeComponent();
			base.CanShutdown = true;
			base.CanStop = true;
			base.CanPauseAndContinue = false;
			base.AutoLog = false;
			base.ServiceName = System.IO.Path.GetFileNameWithoutExtension(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName);
		}

		protected override void OnStart(string[] args)
		{
			try
			{
				Process pro = new Process();
				pro.StartInfo.UseShellExecute = true;
				pro.StartInfo.WorkingDirectory = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase;
				pro.StartInfo.FileName = startPath;  //strBatPath是bat文件路径
				pro.StartInfo.CreateNoWindow = true;
				pro.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;
				pro.Start();
				pro.WaitForExit();
			}
			catch (Exception ex)
			{
				string logFile = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + base.ServiceName + "_logs\\" + DateTime.Now.ToString("yyyy-MM-dd") + ".txt";
				WriteFile(logFile, string.Format("{0} [error] {1} [{2}]\r\n", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), ex.Message, startPath), "UTF-8", false);
			}
		}

		protected override void OnStop()
		{
			try
			{
				Process pro = new Process();
				pro.StartInfo.UseShellExecute = true;
				pro.StartInfo.WorkingDirectory = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase;
				pro.StartInfo.FileName = stopPath;  //strBatPath是bat文件路径
				pro.StartInfo.CreateNoWindow = true;
				pro.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;
				pro.Start();
				pro.WaitForExit();
			}
			catch (Exception ex)
			{
				string logFile = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + base.ServiceName + "_logs\\" + DateTime.Now.ToString("yyyy-MM-dd") + ".txt";
				WriteFile(logFile, string.Format("{0} [error] {1} [{2}]\r\n", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), ex.Message, stopPath), "UTF-8", false);
			}
		}

		public static Boolean WriteFile(String filePath, String content, String charsetName, Boolean overwrite)
		{
			String folder = Path.GetDirectoryName(filePath);// 取得当前路径下全路径（不以/结尾的无法区分是否文件夹，则取得父路径）
			CreateFolder(folder);
			if (!Directory.Exists(filePath) || !File.Exists(filePath))
			{
				String parentFilePath = (new FileInfo(filePath)).DirectoryName;
				if (!Directory.Exists(parentFilePath))
				{
					Directory.CreateDirectory(parentFilePath);// 创建所属目录
				}
			}
			else if (!overwrite || Directory.Exists(filePath))// 不可覆盖或是目录时
			{
				return false;
			}
			Encoding enc = charsetName.ToLower().Equals("utf-8") ? new UTF8Encoding(false) : Encoding.GetEncoding(charsetName);
			using (StreamWriter sw = new StreamWriter(filePath, true, enc))
			{
				sw.Write(content);
				sw.Close();
			}
			return true;
		}
		public static void CreateFolder(string filePath)
		{
			if (!Directory.Exists(filePath))
			{
				Directory.CreateDirectory(filePath);
			}
		}
	}
}
